# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import enum
from datetime import datetime
from typing import NamedTuple


class BacklogLocation(enum.Enum):
    """A location with respect to the message backlog. BEGINNING refers to the
    location of the oldest retained message. END refers to the location past
    all currently published messages, skipping the entire message backlog."""

    BEGINNING = 0
    END = 1


class PublishTime(NamedTuple):
    """The publish timestamp of a message."""

    value: datetime


class EventTime(NamedTuple):
    """A user-defined event timestamp of a message."""

    value: datetime
